from flask import Blueprint, request, jsonify
from flask_jwt_extended import create_access_token

from middlewares.validate_request import validate_request
from dtos.user_dtos import RegisterUserDTO, LoginUserDTO
from repos.user_repo import UserRepo
from models.user_model import User

user_blueprint = Blueprint('user', __name__)

@user_blueprint.route('/register', methods=['POST'])
@validate_request(RegisterUserDTO)
def register_user():
    data = request.json
    if UserRepo.find_by_username(data['username']):
        return jsonify({"error": "User already exists"}), 409
    user = User(**data)
    UserRepo.insert(user)
    return jsonify({"message": "User registered successfully"})

@user_blueprint.route('/login', methods=['POST'])
@validate_request(LoginUserDTO)
def login_user():
    data = request.json
    user = UserRepo.find_by_username(data['username'])
    if user:
        access_token = create_access_token(identity=user['username'])
        return jsonify(access_token=access_token)
    return jsonify({"error": "Invalid credentials"}), 401